@@ -1,5 +1,7 @@
 #include "rpmsystem-py.h"
 
+#include <fcntl.h>
+
 #include <rpm/rpmlib.h>	/* rpmReadPackageFile, headerCheck */
 #include <rpm/rpmtag.h>
 #include <rpm/rpmpgp.h>
@@ -12,13 +14,11 @@
 #include "rpmkeyring-py.h"
 #include "rpmfi-py.h"	/* XXX for rpmfiNew */
 #include "rpmmi-py.h"
+#include "rpmii-py.h"
 #include "rpmps-py.h"
 #include "rpmte-py.h"
-
 #include "rpmts-py.h"
 
-#include "debug.h"
-
 /** \ingroup python
  * \name Class: Rpmts
  * \class Rpmts
@@ -96,7 +96,7 @@
  * @return	None
  *
  * - ts.setFlags(transFlags) Set transaction set flags.
- * @param transFlags - bit(s) to controll transaction operations. The
+ * @param transFlags - bit(s) to control transaction operations. The
  *		following values can be logically OR'ed together:
  *	- rpm.RPMTRANS_FLAG_TEST - test mode, do not modify the RPM
  *		database, change any files, or run any package scripts
@@ -162,7 +162,7 @@ static void die(PyObject *cb)
     if ((r = PyObject_Repr(cb)) != NULL) { 
 	pyfn = PyBytes_AsString(r);
     }
-    fprintf(stderr, _("error: python callback %s failed, aborting!\n"), 
+    fprintf(stderr, "FATAL ERROR: python callback %s failed, aborting!\n", 
 	    	      pyfn ? pyfn : "???");
     rpmdbCheckTerminate(1);
     exit(EXIT_FAILURE);
@@ -187,6 +187,24 @@ rpmts_AddInstall(rpmtsObject * s, PyObject * args)
     return PyBool_FromLong((rc == 0));
 }
 
+static PyObject *
+rpmts_AddReinstall(rpmtsObject * s, PyObject * args)
+{
+    Header h = NULL;
+    PyObject * key;
+    int rc;
+
+    if (!PyArg_ParseTuple(args, "O&O:AddReinstall", 
+			  hdrFromPyObject, &h, &key))
+	return NULL;
+
+    rc = rpmtsAddReinstallElement(s->ts, h, key);
+    if (key && rc == 0) {
+	PyList_Append(s->keyList, key);
+    }
+    return PyBool_FromLong((rc == 0));
+}
+
 static PyObject *
 rpmts_AddErase(rpmtsObject * s, PyObject * args)
 {
@@ -278,6 +296,14 @@ rpmts_Clean(rpmtsObject * s)
     Py_RETURN_NONE;
 }
 
+static PyObject *
+rpmts_Clear(rpmtsObject * s)
+{
+    rpmtsEmpty(s->ts);
+
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 rpmts_OpenDB(rpmtsObject * s)
 {
@@ -349,7 +375,7 @@ rpmts_HdrFromFdno(rpmtsObject * s, PyObject *arg)
     	return NULL;
 
     Py_BEGIN_ALLOW_THREADS;
-    rpmrc = rpmReadPackageFile(s->ts, rpmfdGetFd(fdo), "rpmts_HdrFromFdno", &h);
+    rpmrc = rpmReadPackageFile(s->ts, rpmfdGetFd(fdo), NULL, &h);
     Py_END_ALLOW_THREADS;
     Py_XDECREF(fdo);
 
@@ -517,15 +543,8 @@ rpmtsCallback(const void * hd, const rpmCallbackType what,
 static PyObject *
 rpmts_Problems(rpmtsObject * s)
 {
-    PyObject *problems = PyList_New(0);
     rpmps ps = rpmtsProblems(s->ts);
-    rpmpsi psi = rpmpsInitIterator(ps);
-    while (rpmpsNextIterator(psi) >= 0) {
-	PyObject *prob = rpmprob_Wrap(&rpmProblem_Type, rpmpsGetProblem(psi));
-	PyList_Append(problems, prob);
-	Py_DECREF(prob);
-    }
-    rpmpsFreeIterator(psi);
+    PyObject *problems = rpmps_AsList(ps);
     rpmpsFree(ps);
     return problems;
 }
@@ -596,7 +615,7 @@ rpmts_Match(rpmtsObject * s, PyObject * args, PyObject * kwds)
 /* XXX lkey *must* be a 32 bit integer, int "works" on all known platforms. */
     int lkey = 0;
     int len = 0;
-    rpmTag tag = RPMDBI_PACKAGES;
+    rpmDbiTagVal tag = RPMDBI_PACKAGES;
     char * kwlist[] = {"tagNumber", "key", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O:Match", kwlist,
@@ -608,6 +627,10 @@ rpmts_Match(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	    lkey = PyInt_AsLong(Key);
 	    key = (char *)&lkey;
 	    len = sizeof(lkey);
+	} else if (PyLong_Check(Key)) {
+	    lkey = PyLong_AsLong(Key);
+	    key = (char *)&lkey;
+	    len = sizeof(lkey);
 	} else if (utf8FromPyObject(Key, &str)) {
 	    key = PyBytes_AsString(str);
 	    len = PyBytes_Size(str);
@@ -635,10 +658,42 @@ exit:
     Py_XDECREF(str);
     return mio;
 }
+static PyObject *
+rpmts_index(rpmtsObject * s, PyObject * args, PyObject * kwds)
+{
+    rpmDbiTagVal tag;
+    PyObject *mio = NULL;
+    char * kwlist[] = {"tag", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&:Keys", kwlist,
+              tagNumFromPyObject, &tag))
+	return NULL;
+
+    /* XXX If not already opened, open the database O_RDONLY now. */
+    if (rpmtsGetRdb(s->ts) == NULL) {
+	int rc = rpmtsOpenDB(s->ts, O_RDONLY);
+	if (rc || rpmtsGetRdb(s->ts) == NULL) {
+	    PyErr_SetString(pyrpmError, "rpmdb open failed");
+	    goto exit;
+	}
+    }
+
+    rpmdbIndexIterator ii = rpmdbIndexIteratorInit(rpmtsGetRdb(s->ts), tag);
+    if (ii == NULL) {
+        PyErr_SetString(PyExc_KeyError, "No index for this tag");
+        return NULL;
+    }
+    mio = rpmii_Wrap(&rpmii_Type, ii, (PyObject*)s);
+
+exit:
+    return mio;
+}
 
 static struct PyMethodDef rpmts_methods[] = {
  {"addInstall",	(PyCFunction) rpmts_AddInstall,	METH_VARARGS,
 	NULL },
+ {"addReinstall",	(PyCFunction) rpmts_AddReinstall,	METH_VARARGS,
+	NULL },
  {"addErase",	(PyCFunction) rpmts_AddErase,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
  {"check",	(PyCFunction) rpmts_Check,	METH_VARARGS|METH_KEYWORDS,
@@ -654,6 +709,9 @@ static struct PyMethodDef rpmts_methods[] = {
   Note: The callback may not be None.\n" },
  {"clean",	(PyCFunction) rpmts_Clean,	METH_NOARGS,
 	NULL },
+ {"clear",	(PyCFunction) rpmts_Clear,	METH_NOARGS,
+"ts.clear() -> None\n\
+Remove all elements from the transaction set\n" },
  {"openDB",	(PyCFunction) rpmts_OpenDB,	METH_NOARGS,
 "ts.openDB() -> None\n\
 - Open the default transaction rpmdb.\n\
@@ -686,8 +744,11 @@ static struct PyMethodDef rpmts_methods[] = {
  {"setKeyring",	(PyCFunction) rpmts_setKeyring,	METH_O, 
 	NULL },
  {"dbMatch",	(PyCFunction) rpmts_Match,	METH_VARARGS|METH_KEYWORDS,
-"ts.dbMatch([TagN, [key, [len]]]) -> mi\n\
+"ts.dbMatch([TagN, [key]]) -> mi\n\
 - Create a match iterator for the default transaction rpmdb.\n" },
+ {"dbIndex",     (PyCFunction) rpmts_index,	METH_VARARGS|METH_KEYWORDS,
+"ts.dbIndex(TagN) -> ii\n\
+- Create a key iterator for the default transaction rpmdb.\n" },
     {NULL,		NULL}		/* sentinel */
 };
 
@@ -714,7 +775,7 @@ static PyObject * rpmts_new(PyTypeObject * subtype, PyObject *args, PyObject *kw
 
 static int rpmts_init(rpmtsObject *s, PyObject *args, PyObject *kwds)
 {
-    char * rootDir = "/";
+    const char * rootDir = "/";
     rpmVSFlags vsflags = rpmExpandNumeric("%{?__vsflags}");
     char * kwlist[] = {"rootdir", "vsflags", 0};
 
